% ======================================================================
% 0. This are the discretization step: the resulting models will be 
%    computed in the interval [0:dx:Rmax=1.05]
% ======================================================================
clear all ; timer=tic;
dx=1e-7;
flag_cgs=1; % choice of units, SI per default.

% ======================================================================
% 1. we load the spline models and set up the constant.
% ======================================================================
% we load the splines for the density and adiabatic index.
load splines_gamma_adiabatic.mat
load modelS_origin.mat

if(flag_cgs == 0) % using the SI units.
  gravity    = 6.67430e-11     ; % gravity constant.
  Rsun       = 696000.e3       ; % Sun radius in meters.
  load splines_density_SI.mat	
else
  gravity    = 6.67430e-8      ; % gravity constant.
  Rsun       = 696000.e5       ; % Sun radius in centimeters.
  load splines_density_CGS.mat	
  % convert model S units given in (kg.m-3) to convert in (g.cm-3)
  %                   = x 1000 / (1000^2) = / 1e3
  modelS_coeff.rho = modelS_coeff.rho / 1e3;
  % also changes the fluid pressure originally in kg.s^{-2}.m^{-1}
  modelS_coeff.p   = modelS_coeff.p   * 1e1; % = x 1000 / 100
end

% we also need the derivatives
spline_dlogrho  = fnder(spline_logrho, 1); % still a type, order 1 derivative
spline_d2logrho = fnder(spline_logrho, 2); % still a type, order 2 derivative

x = [0:dx:Rmax] ; 
nx= length(x) ;

fprintf('----------------------------------------------------\n')
fprintf('- Computation of the Solar background models \n')
fprintf('  interval           : [0,  %f] \n',Rmax)
fprintf('  discretization step: %f \n',dx)
fprintf('  n points           : %d \n',nx)
if(flag_cgs == 0) % using the SI units.
  fprintf('  Using the SI units: kg and m \n')
else
  fprintf('  Using the CGS units: g and cm \n')
end
fprintf('----------------------------------------------------\n')


% ======================================================================
% 2. we compute \phi0' and \phi0" from the density
% ======================================================================
dphi0    = func_dphi0(spline_logrho,Rmax,gravity,dx) ;
dphi0    = dphi0.'; 
% compute the derivatives \phi0'' = 4 pi G rho - 2./r phi0'
% with limit of (4 pi G rho(0) / 3) in 0
d2phi0 = zeros(nx,1) ; 
d2phi0(1) = 4./3.*pi.*gravity * exp(ppval(spline_logrho,0));
rho = exp(ppval(spline_logrho,x));
for ix=2:nx
  xloc = x(ix) ;
  d2phi0(ix) = 4.*pi.*gravity*rho(ix) - 2./xloc .* dphi0(ix) ;
end
% we now get \phi0''' = 4 pi G rho' - 2 d2phi/r + 2 dphi/r²
temp = ppval(spline_dlogrho,x) ; % = rho' / rho
drho = temp .* rho ; clear temp
drho = drho.' ; rho  = rho.'; 
d3phi0 = 4.*pi.*gravity.*drho - 2.0.*d2phi0./x.' + 2.*dphi0./((x.').^2) ;
% organize these
gravity_potential.x      = x.'    ;
gravity_potential.dphi0  = dphi0  ; clear dphi0  
gravity_potential.d2phi0 = d2phi0 ; clear d2phi0 
gravity_potential.d3phi0 = d3phi0 ; clear d3phi0 
clear ix xloc 
fprintf('-  computation of the gravity potential [0  Rmax] OK \n')

% ======================================================================
% 3. We compute the pressure derivatives in [0, Rs]
%    p0' = - \rho \phi_0'
%    p0" = - \rho \phi_0'' - \rho'\phi_0'
% ======================================================================
x_rs = [0:dx:Rs] ; nx_rs = length(x_rs) ;
dp0  = -  rho(1:nx_rs).*gravity_potential.dphi0 (1:nx_rs); 
d2p0 = -  rho(1:nx_rs).*gravity_potential.d2phi0(1:nx_rs) ...
       - drho(1:nx_rs).*gravity_potential.dphi0 (1:nx_rs); 
% ======================================================================
% We compute the pressure using integration: 
%    p0  =  p0(rs) + \integral_r^rs \rho \phi_0'.
% ======================================================================
p0  = zeros(nx_rs,1) ; 
% it is the integral from r to Rs, so revert from usual
myint = zeros(nx_rs,1)  ; 
for ix=nx_rs-1:-1:1
  dist      = x_rs(ix+1) - x_rs(ix); 
  myint(ix) = myint(ix+1) + dist/2.0 * (rho(ix+1)*gravity_potential.dphi0(ix+1) ...
                                      + rho(ix)  *gravity_potential.dphi0(ix)) ; 
end 
p_end = (modelS_coeff.p(end) ./ (Rsun.^2)) ; 

p0 = myint + p_end ; % add p0(rs) to the integral using modelS.
clear p_end
clear dist ix  myint
fprintf('-  computation of the fluid pressure    [0  Rs]   OK \n')

% ======================================================================
% 4. We compute Ehe and \tilde{Ehe}
%    it is 0 between [0, Rs]
%    then, we only need to have p0(ra) to evaluate everywhere else
% ======================================================================
% compute the values in Ra
ind_Ra = floor(Ra/dx) + 1 ;
temp   = ppval(spline_d2logrho,x).' ; % = (rho'' rho - rho' rho') / rho.²
d2rho  = (rho.*rho.*temp + drho.*drho) ./ rho;  clear temp
frak_a =   rho(ind_Ra)*gravity_potential.dphi0(ind_Ra) ; 
frak_b =  drho(ind_Ra)*gravity_potential.dphi0(ind_Ra) + rho(ind_Ra) * gravity_potential.d2phi0(ind_Ra) ;
frak_c = d2rho(ind_Ra)*gravity_potential.dphi0(ind_Ra) + 2.0*drho(ind_Ra)*gravity_potential.d2phi0(ind_Ra) + rho(ind_Ra)*gravity_potential.d3phi0(ind_Ra) ;
clear spline_d2logrho spline_dlogrho

% compute MRa= \int_rs^ra \rho \partial \phi_0 dx.
MRa = 0.0;
for ix=nx_rs+1:ind_Ra
  dist   = x(ix) - x(ix-1); 
  MRa    = MRa + dist/2.0 * (rho(ix-1)*gravity_potential.dphi0(ix-1) + rho(ix)*gravity_potential.dphi0(ix)) ; 
end 
clear dist ix

% compute constants Q and C 
dist= Ra - Rs ; 
Q0  = dist.^4 ./  4. ; 
Q1  =-dist.^5 ./ 20. ;
Q2  = dist.^6 ./ 60. ;
C10 = (frak_a*alpha_a + frak_b) ./ (dist.^3) ; 
C11 =-(alpha_a + 3./dist) ; 
C20 =(-frak_a * (alpha_a.^2/2 + 3.*alpha_a/dist)  + frak_c/2.  - 3.*frak_b/dist) * 1./(dist.^3);
C21 =(alpha_a.^2/2. + 6./dist.^2 + 3.*alpha_a/dist);  
p0Rs= p0(end); % it currently stops in Rs.

% get a0, a1, a2: C.22 C.24, C.34
a0 = (MRa + frak_a / alpha_a - Q1*C10 - Q2*C20 - p0Rs) ./ (Q0 + Q1*C11 + Q2*C21 + dist.^3/alpha_a) ;
a1 =  C10 + C11*a0 ; 
a2 =  C20 + C21*a0 ;
clear dist 

% compute \tilde{Ehe}
Ehetilda = zeros(nx,1); 
for ix=nx_rs+1:ind_Ra
  distRa  = x(ix) - Ra;
  distRs  = x(ix) - Rs;
  Ehetilda(ix) = (a0 + a1*distRa + a2*distRa.^2) * (distRs.^3);
end  
clear xpos ix distRa distRs 
clear Q0 Q1 Q2 dist C10 C11 C20 C21 MRa frak_a frak_b frak_c
fprintf('-  computation of tilde{Ehe}            [0  Ra]   OK \n')

% ======================================================================
% 5. Using Ehetilda up until Ra, we compute p0 in the transition
%    and for r > Ra
%    we have in the transition: 
%      p0 = - M + Q + p0(rs)
%    with 
%      M = int_rs^r rho \phi_0' .
%      Q = int_rs^r Ehetilda         .
% ======================================================================
M      = zeros(nx,1);
Q      = zeros(nx,1);
p0_full= zeros(nx,1);
p0_full(1:nx_rs)=p0;
for ix=nx_rs+1:ind_Ra
  dist   = x(ix) - x(ix-1); 
  M  (ix)= M  (ix-1) + dist/2.0 * (rho(ix-1)*gravity_potential.dphi0(ix-1) + rho(ix)*gravity_potential.dphi0(ix)) ; 
  Q  (ix)= Q  (ix-1) + dist/2.0 * (Ehetilda(ix-1) + Ehetilda(ix)) ; 
  % extend p0 
  p0_full(ix) = -M(ix) + Q(ix) + p0Rs;
end 
% after Ra, it is given by
% p0 = p0(ra) exp(-alpha (r-ra))
for ix=ind_Ra+1:nx
  dist   = x(ix) - x(ind_Ra); 
  p0_full(ix) = p0_full(ind_Ra) * exp(-alpha_a*(dist));
end 
clear ix dist M Q p0Rs
fprintf('-  computation of the fluid pressure    [Rs Rmax] OK \n')

% ======================================================================
% 6. Using p0 up until Rmax, we compute Ehetilda after Ra.
%    using 
%      Ehetilda = \rho \phi_0' - p0(ra) alpha exp(-alpha (r-ra)) for r > Ra
% ======================================================================
for ix=ind_Ra+1:nx
  dist    = x(ix) - x(ind_Ra); 
  Ehetilda(ix) = rho(ix)*gravity_potential.dphi0(ix) ...
               - p0_full(ind_Ra) * alpha_a * exp(-alpha_a * dist);
end 
clear ix dist 
fprintf('-  computation of tilde{Ehe}            [Ra Rmax] OK \n')

% ======================================================================
% 7. Using p0 we compute p0' and p0'' for r > rs 
% ======================================================================
dp0_full = zeros(nx,1);
d2p0_full= zeros(nx,1);
 dp0_full(1:nx_rs) =  dp0;
d2p0_full(1:nx_rs) = d2p0; 
% transition 
for ix=nx_rs+1:ind_Ra
 dp0_full(ix) = -rho(ix)*gravity_potential.dphi0(ix) + Ehetilda(ix) ;
 % using the derivatives of Ehetilda=(a0 + a1*distRa + a2*distRa.^2) * (distRs.^3);
 distRa  = x(ix) - Ra;
 distRs  = x(ix) - Rs;
 dEhetilda  = 3.*(distRs.^2) * (a0 + a1*distRa + a2*distRa.^2) ...
            +    (distRs.^3) * (a1 + 2.*a2*distRa);
 d2p0_full(ix) = -drho(ix)*gravity_potential.dphi0(ix) - rho(ix)*gravity_potential.d2phi0(ix) + dEhetilda;
end 
clear dEhetilda
% in the atmosphere, p0 is given by the exponential: p0_full(ind_Ra) * exp(-alpha*(dist));
for ix=ind_Ra+1:nx
       dist    = x(ix) - x(ind_Ra); 
  dp0_full(ix) = - alpha_a   * p0_full(ind_Ra) * exp(-alpha_a * dist);
 d2p0_full(ix) =   alpha_a.^2* p0_full(ind_Ra) * exp(-alpha_a * dist);
end 
clear ix distRa distRs dist dp0 d2p0 p0 
% organize the fluid pressure
pressure.x    = x          ;
pressure.p0   = p0_full    ; clear p0_full  
pressure.dp0  = dp0_full   ; clear dp0_full 
pressure.d2p0 = d2p0_full  ; clear d2p0_full
clear a0 a1 a2 nx_rs x_rs
fprintf("-  computation of the p' and p''        [Rs Rmax] OK \n")

% ======================================================================
% 8. deduce the velocity and Ehe parameters.
% ======================================================================
adiabatic = ppval(spline_adiabatic,x).' ; 
velocity.x   = x ; 
velocity.cp  = sqrt( adiabatic .*   pressure.p0 ./ rho);
velocity.cp(ind_Ra+1:end) = velocity.cp(ind_Ra);

Ehe.x        = x ; 
Ehe.Ehe      = Ehetilda ./ (adiabatic .* pressure.p0) ;
Ehe.Ehetilda = Ehetilda ; clear Ehetilda 
% after Ra
for ix=ind_Ra+1:nx
  Ehe.Ehe(ix) =  gravity_potential.dphi0(ix)./(velocity.cp(ix).^2) ...
              -  alpha_a ./ adiabatic(ix);
end 
clear adiabatic ind_Ra ix nx x
clear rho drho d2rho 
fprintf('-  computation of cp and Ehe            [0  Rmax] OK \n')
fprintf('----------------------------------------------------\n')
fprintf('- Computational time: %f seconds \n', toc(timer))
fprintf('----------------------------------------------------\n')
clear timer

return
